// Copyright (C) 2004 MySQL AB
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License version 2 as published by
// the Free Software Foundation
//
// There are special exceptions to the terms and conditions of the GPL 
// as it is applied to this software. View the full text of the 
// exception in file EXCEPTIONS in the directory of this software 
// distribution.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 

using System;
using System.Data;
using MySql.Data.MySqlClient;

namespace MySql.Data.Types
{
	/// <summary>
	/// Summary description for MySqlByte.
	/// </summary>
	internal class MySqlByte : MySqlNumber
	{
		private sbyte	mValue;

		public MySqlByte() : base()
		{
			dbType = DbType.SByte;
			mySqlDbType = MySqlDbType.Byte;
		}

		public MySqlByte(MySqlDbType type) : this()
		{
			mySqlDbType = type;
		}

		internal override void Serialize(PacketWriter writer, bool binary, object value, int length)
		{
			sbyte v = Convert.ToSByte( value );
			if (binary)
				writer.WriteByte( (byte)v );
			else
				writer.WriteStringNoNull( v.ToString() );
		}


		public override byte[] ToBytes()
		{
			return new byte[1] { (byte)mValue };
		}

		internal override object ValueAsObject
		{
			get { return mValue; }
			set { mValue = Convert.ToSByte(value); } 
		}

		public sbyte Value
		{
			get { return mValue; }
			set { mValue = value; }
		}

		internal override Type SystemType
		{
			get { return typeof(SByte); }
		}

		internal override string GetMySqlTypeName()
		{ 
			return "TINYINT"; 
		}

		internal override void SetData(PacketReader reader, long length)
		{
			if (length == -1)
				mValue = (sbyte)reader.ReadByte();
			else 
				mValue = SByte.Parse( reader.ReadString( length ) );
		}

		internal override void Skip(PacketReader reader)
		{
			reader.ReadByte();
		}


	}
}
