// Copyright (C) 2004 MySQL AB
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License version 2 as published by
// the Free Software Foundation
//
// There are special exceptions to the terms and conditions of the GPL 
// as it is applied to this software. View the full text of the 
// exception in file EXCEPTIONS in the directory of this software 
// distribution.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 

using System;
using System.Data;
using MySql.Data.MySqlClient;

namespace MySql.Data.Types
{
	/// <summary>
	/// Summary description for MySqlDouble.
	/// </summary>
	internal class MySqlDouble : MySqlNumber
	{
		private double	mValue;

		public MySqlDouble() : base()
		{
			dbType = DbType.Double;
			mySqlDbType = MySqlDbType.Double;
		}

		public MySqlDouble(double val) : this()
		{
			mValue = val;
		}

		internal override void Serialize(PacketWriter writer, bool binary, object value, int length)
		{
			double v = Convert.ToDouble(value);
			if (binary)
				writer.Write( BitConverter.GetBytes( v ) );
			else
				writer.WriteStringNoNull( v.ToString(numberFormat) );
		}


		public override byte[] ToBytes()
		{
			return BitConverter.GetBytes( mValue );
		}

		internal override object ValueAsObject
		{
			get { return mValue; }
			set { mValue = Convert.ToDouble(value); }
		}

		public double Value
		{
			get { return mValue; }
			set { mValue = value; }
		}

		internal override Type SystemType
		{
			get { return typeof(Double); }
		}

		internal override string GetMySqlTypeName()
		{
			return "DOUBLE";
		}

		internal override void SetData(PacketReader reader, long length)
		{
			if (length == -1) 
			{
				byte[] b = new byte[8];
				reader.Read( ref b, 0, 8 );
				mValue = BitConverter.ToDouble( b, 0 );
			}
			else 
			{
				string value = reader.ReadString( length );
				mValue = Double.Parse( value, numberFormat );
			}
		}

		internal override void Skip(PacketReader reader)
		{
			reader.Skip( 8 );
		}
	}
}
